user www-data;
worker_processes auto;

error_log /var/log/nginx/error.log;
# pid /var/run/nginx.pid;

events {
    worker_connections 1024;
    use epoll;
}

http {
    upstream frontends {
        server 127.0.0.1:8888;
    }

    include /etc/nginx/mime.types;
    default_type application/octet-stream;

    access_log /var/log/nginx/access.log;

    keepalive_timeout 65;
    proxy_read_timeout 200;
    sendfile on;
    tcp_nopush on;
    tcp_nodelay on;
    gzip on;
    gzip_min_length 1000;
    gzip_proxied any;
    gzip_types text/plain text/html text/css text/xml
               application/x-javascript application/xml
               application/atom+xml text/javascript;

    # Only retry if there was a communication error, not a timeout
    # on the Tornado server (to avoid propagating "queries of death"
    # to all frontends)
    proxy_next_upstream error;

    server {
        listen 80;
        server_name {{server_name}} www.{{server_name}};

        # Allow file uploads
        client_max_body_size 1G;

        location = /favicon.ico {
            rewrite (.*) /static/images/favicon.ico;
        }
        location = /robots.txt {
            rewrite (.*) /static/robots.txt;
        }
        location = /sitemap.xml {
            rewrite (.*) /static/sitemap.xml;
        }

        location / {
            proxy_pass_header Server;
            proxy_set_header Host $http_host;
            proxy_redirect off;
            proxy_set_header X-Real-IP $remote_addr;
            proxy_set_header X-Scheme $scheme;
            proxy_pass http://frontends;
        }
    }
}
