import faiss
import os

from common.ivf_train import dimension

os.environ['KMP_DUPLICATE_LIB_OK'] = 'True'

from pathlib import Path


class FaissIndex():
    def __init__(self, index_name):
        self.index = None
        self.index_path = os.path.join(Path.home(), index_name + '.index')

        if os.path.isfile(self.index_path):
            self.index = faiss.read_index(self.index_path, faiss.IO_FLAG_ONDISK_SAME_DIR)
        else:
            self.index = faiss.index_factory(dimension, "IDMap,Flat", faiss.METRIC_L2)

    def add(self, vector, iid):
        self.index.add_with_ids(vector, iid)

    def rate(self, vector, ncount=5):
        d, i = self.index.search(vector, ncount)
        return d, i

    def save(self):
        faiss.write_index(self.index, self.index_path)


faiss_index = FaissIndex("global")
