import os

import faiss
import numpy as np
from faiss.contrib.ondisk import merge_ondisk

from settings import FAISS_DATA_ROOT_PATH

dimension = 1024

os.makedirs(FAISS_DATA_ROOT_PATH, exist_ok=True)
_f_index = lambda name: os.path.join(FAISS_DATA_ROOT_PATH, 'train', name)


class FaissTrainer:
    trained_index_fn = _f_index('trained.index')
    batch_train_index_fn = lambda _, x: _f_index(f'batch_{x}.index')
    merged_index_fn = _f_index('merged_index.ivfdata')
    populated_index_fn = _f_index('populated.index')

    def __init__(self, index_name):
        self.batch_pathes = []
        self.count_ids = 0
        self._index_name = index_name

    def initial_train(self, initial_data: np.ndarray):
        assert initial_data.shape[1] == dimension, 'dimension of trained initial vector not equals with index dimension'
        index = faiss.index_factory(dimension, self._index_name)
        print(f'initial train data')
        index.train(initial_data)
        print(f'initial write trained index to {self.trained_index_fn}')
        faiss.write_index(index, self.trained_index_fn)

    def append_trained_batch_element(self, batch_el: np.ndarray):
        index = faiss.read_index(self.trained_index_fn)
        index.add_with_ids(np.array([batch_el]), np.arange(self.count_ids, self.count_ids + 1))
        self.count_ids += 1
        b_index_path = self.batch_pathes[-1]
        faiss.write_index(index, b_index_path)
        print(f'trained batch appended to {b_index_path}')

    def add_trained_batch(self, batch: np.ndarray):
        index = faiss.read_index(self.trained_index_fn)
        index.add_with_ids(batch, np.arange(self.count_ids, self.count_ids + batch.shape[0]))
        self.count_ids += batch.shape[0]
        b_index_path = self.batch_train_index_fn(len(self.batch_pathes) + 1)
        faiss.write_index(index, b_index_path)
        self.batch_pathes.append(b_index_path)
        print(f'trained batch added {b_index_path}')

    def merge(self):
        index = faiss.read_index(self.trained_index_fn)

        print(f'merge batches')
        merge_ondisk(
            index,
            self.batch_pathes,
            self.merged_index_fn
        )
        print(f'write populated index')
        faiss.write_index(index, self.populated_index_fn)
        return self.populated_index_fn

