import os.path
import config
import tornado.httpserver
import tornado.ioloop
import tornado.web
from tornado.options import options
from tornado_swagger.setup import setup_swagger

from controllers.faiss_service import AddController
from controllers.faiss_service import RateController


class FAISSApplication(tornado.web.Application):
    _routes = [
        tornado.web.url(r"/add", AddController),
        tornado.web.url(r"/rate", RateController),
    ]

    def __init__(self):
        settings = dict(login_url=options.login_url,
                        cookie_secret=options.cookie_secret,
                        users_secret=options.users_secret,
                        template_path=os.path.join(os.path.dirname(__file__), "templates"),
                        static_path=os.path.join(os.path.dirname(__file__), "static"),
                        xsrf_cookies=False,
                        debug=options.debug)

        setup_swagger(
            self._routes,
            swagger_url="/doc",
            api_base_url="/",
            api_version="0.0.1",
            title="GRS-service api",
        )
        super(FAISSApplication, self).__init__(self._routes, **settings)


def init_app():
    return FAISSApplication()


def main():
    tornado.options.parse_command_line()

    application = init_app()
    http_server = tornado.httpserver.HTTPServer(application, max_body_size=1024 * 1024 * 1024)
    http_server.listen(options.port)
    loop = tornado.ioloop.IOLoop.current()
    try:
        print(f'start service on {options.port}')
        loop.start()
    except Exception as e:
        print(f'exc {e}')
    finally:
        http_server.stop()
        loop.stop()


if __name__ == '__main__':
    main()
