import torch
import torch.nn as nn
import torchvision.models as models
import torchvision.transforms as transforms
import numpy as np


class ImageProcessor():
    RESNET_OUTPUT_SIZES = {
        'resnet18': 512,
        'resnet50': 2048,
    }

    def __init__(self, model_name='resnet18', layer_name='default', layer_output_size=512):

        self.device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
        self.model_name = model_name

        self.model = getattr(models, model_name)(pretrained=True)
        if layer_name == 'default':
            self.layer = self.model._modules.get('avgpool')
            self.layer_output_size = self.RESNET_OUTPUT_SIZES[model_name]
        else:
            self.layer = self.model._modules.get(layer_name)
            self.layer_output_size = layer_output_size

        self.model = self.model.to(self.device)

        self.model.eval()

        self.scaler = transforms.Resize((224, 224))
        self.normalize = transforms.Normalize(mean=[0.485, 0.456, 0.406],
                                              std=[0.229, 0.224, 0.225])
        self.to_tensor = transforms.ToTensor()

    def get_vector(self, img) -> np.ndarray:
        image = self.normalize(self.to_tensor(self.scaler(img))).unsqueeze(0).to(self.device)

        embedding = torch.zeros(1, self.layer_output_size, 1, 1)

        def copy_data(m, i, o):
            embedding.copy_(o.data)

        h = self.layer.register_forward_hook(copy_data)
        h_x = self.model(image)

        h.remove()

        return embedding.numpy()[:, :, 0, 0]
