import io
import os
import random
import time
import unittest
from unittest import skipIf, skip

import faiss
import numpy as np
from PIL import Image

from common.ivf_train import FaissTrainer, dimension
from tests.additional.geo_processor import append_geo
from tests.additional.image_processor import ImageProcessor

image_processor = ImageProcessor()

dataset_fp = '/datasets/oids2019'


def make_vector(fn, lat, lon, accuracy) -> np.ndarray:
    with open(fn, 'rb') as f:
        vec = image_processor.get_vector(Image.open(io.BytesIO(f.read())))
    vec = append_geo(vec, lat, lon, accuracy)
    return vec


def rand_l(n):
    return (random.random() * 2 * n) - n


def rand_h(n):
    return n / 2 + (random.random() * (n / 2))


def prepare_dataset():
    train, test = [], []
    ptest, ptrain = os.path.join(dataset_fp, 'test'), os.path.join(dataset_fp, 'train')
    for f in os.listdir(ptest):
        test.append((os.path.join(ptest, f), rand_l(90), rand_l(180), rand_h(30)))

    for f in os.listdir(ptrain):
        train.append((os.path.join(ptrain, f), rand_l(90), rand_l(180), rand_h(30)))

    return train, test


def generate_data(batch_size):
    tmp_dir = os.path.join(dataset_fp, 'tmp')
    os.makedirs(tmp_dir, exist_ok=True)
    for fn in os.listdir(tmp_dir):
        if 'vectors' in fn and str(batch_size) in fn:
            vectors = np.load(os.path.join(tmp_dir, fn), allow_pickle=True)
            print(f'get cached')
            yield vectors

    train, test = prepare_dataset()
    vectors = np.zeros((batch_size, 1024))
    strt = time.time()
    i = 0
    c = 0
    for img_f, lat, lon, acc in train + test:
        vector = make_vector(img_f, lat, lon, acc)
        vectors[i] = vector
        i += 1
        if i == batch_size:
            print(f'{i} at {time.time() - strt}')
            vectors = vectors.astype('float32')
            vectors.dump(os.path.join(tmp_dir, f'{c}_{batch_size}.vectors'))
            yield vectors
            vectors = np.zeros((batch_size, 1024))
            i = 0
            c += 1

    print(f'Vectors {vectors.shape[0]} made by {time.time() - strt}')


def generate_random_data(batch_size):
    for i in range(10):
        yield np.random.sample((batch_size, dimension)).astype('float32')


def check_index(index, test_data=generate_random_data(20)):
    index.nprobe = 16
    for data in test_data:
        for vec in data:
            D, I = index.search(vec.reshape(1, 1024), 30)
            assert D.shape == (1, 30)
            assert I.shape == (1, 30)


class FaissTrainerTest(unittest.TestCase):

    @skip
    def test_index_make(self):
        index_name = "IVF256,PQ64"
        # index_name = "IVF512,PQ8"

        index_maker = FaissTrainer(index_name)
        batch_size = 9984
        # batch_size = 19968
        for i, data in enumerate(generate_random_data(batch_size)):
            if i == 0:
                index_maker.initial_train(data)
            elif i == 1:
                index_maker.add_trained_batch(data)
            elif i == 2:
                index_maker.append_trained_batch_element(data[0])
            else:
                index_maker.add_trained_batch(data)

        index_fn = index_maker.merge()
        index = faiss.read_index(index_fn)
        check_index(index, generate_random_data(100))
