import unittest
from unittest import skipIf

import numpy as np

from common.storages import AccumulationStorage, get_mongo_client

test_db_name = 'faiss-service-test'


@skipIf(get_mongo_client() == None, 'skip if mongo not connected')
class AccumulationStorageTest(unittest.TestCase):
    def setUp(self) -> None:
        self.storage = AccumulationStorage(name=test_db_name)

    def tearDown(self) -> None:
        self.storage.mongo.drop_database(test_db_name)

    def test_accumulation(self):
        prev_cnt = self.storage.counter
        for i in range(1000):
            self.storage.accumulate(np.array([1, 2, 3, 4, 5]).astype('float32'), i % 13)
        self.assertEqual(self.storage.counter, prev_cnt + 1000)

        for el in self.storage.iterate():
            self.assertEqual(el['vector'].tolist(), [1., 2., 3., 4., 5.])
